
#include "loader/loader_api.h"
#include "rom/myRom.h"
#include "rom/keyboard.h"
#include "constants.h"
#include "scene/scene.h"
#include "engine/engine.h"
#include "util.h"
#include "raycast/raycast.h"

#include "raster/raster.h"
#include "3d/3d.h"

#define ROT_ANGLE_STEP 16


unsigned char hasKey        = 0;
unsigned char hasArbalete   = 0;
signed char health          = 100;
unsigned char gameOngoing   = 0;
unsigned char isEquiped     = 0;

extern signed  char innerDoorData[]; // state, variable index in scene
extern signed  char outterDoorData[]; // state, variable index in scene
extern signed  char soldierData[];

// #define CHANGE_INK_TO_RED	            1		
// #define CHANGE_INK_TO_GREEN	            2		
// #define CHANGE_INK_TO_YELLOW	        3		
// #define CHANGE_INK_TO_BLUE	            4		
// #define HIRES_50Hz	30
// #define LORES_SCREEN_WIDTH 40
// void prepare_screen_layout() {
//     int i,j;
//     int c;

//     // int ii, jj;

//     // memoryset((unsigned char*)0xa000,64,40*200);
//     MEMCPY((unsigned char*)0x9800,(unsigned char*)0xb400,0xbb80-0xb400);
//     // memoryset((unsigned char*)0xbb80,'A',40*25);
//     MEMSET((unsigned char*)0xa000,64,5439);

//     for (i=0xbb80;i<0xbfe0;i++)
//     {
// 		pk(i,' ');
//     }

//     for (i=0; i < 132;  i+=3){
//         poke (HIRES_SCREEN_ADDRESS+((i)*NEXT_SCANLINE_INCREMENT),CHANGE_INK_TO_RED);
//         poke (HIRES_SCREEN_ADDRESS+((i+1)*NEXT_SCANLINE_INCREMENT),CHANGE_INK_TO_GREEN);
//         poke (HIRES_SCREEN_ADDRESS+((i+2)*NEXT_SCANLINE_INCREMENT),CHANGE_INK_TO_BLUE);
//     }


//     pk(0xa000+132*40+39,26);    // Text mode

//     pk(0xbfdf,HIRES_50Hz);
// }


void onKey(unsigned char c){
    if (c == KEY_UP) {
            forward(); 
            glCamPosX   = rayCamPosX;
            glCamPosY   = rayCamPosY;
            // refreshNeeded   = 1;
    } else if (c == KEY_DOWN) {
            backward();
            glCamPosX   = rayCamPosX;
            glCamPosY   = rayCamPosY;
            // refreshNeeded   = 1;
    } else if (c == KEY_LEFT) {
            rayCamRotZ      += ROT_ANGLE_STEP;
            glCamRotZ       = rayCamRotZ;
            RayLeftAlpha    = rayCamRotZ + HALF_FOV_FIX_ANGLE;
            // refreshNeeded   = 1;
    } else if (c == KEY_RIGHT) {
            rayCamRotZ      -= ROT_ANGLE_STEP; 
            glCamRotZ       = rayCamRotZ;
            RayLeftAlpha    = rayCamRotZ + HALF_FOV_FIX_ANGLE;
            // refreshNeeded   = 1;
    } else if (c == KEY_J) {
            // refreshNeeded           = 1;
            shiftRight();
            glCamPosX   = rayCamPosX;
            glCamPosY   = rayCamPosY;
    } else if (c == KEY_H) {
            // refreshNeeded           = 1;
            shiftLeft();
            glCamPosX   = rayCamPosX;
            glCamPosY   = rayCamPosY;
    } else if (c == KEY_SPACE) {
        if (hasArbalete != 0) missileNew();
    } else if (c == KEY_E) {
        if ((hasArbalete != 0) && (isEquiped == 0)) isEquiped = 1;
        else isEquiped = 0;
    }
}



void onIT(){;}

void initCamera(){
    rayCamPosX               = 71;
    rayCamPosY               = 9; 
    rayCamRotZ               = 0x80;
    RayLeftAlpha            = rayCamRotZ + HALF_FOV_FIX_ANGLE;

    glCamPosX   = rayCamPosX;
    glCamPosY   = rayCamPosY;
    glCamPosZ   = 0;
    glCamRotZ   = rayCamRotZ;


}

unsigned char previousArea;
unsigned char currentArea = 255;

void updateCurrentArea (){

    previousArea = currentArea;

    currentArea = 0;
    if (abs(rayCamPosX) <= 21) {
        if ((rayCamPosY >= 24) && (rayCamPosY < 60) ) {
            currentArea = 1;
        } else if ((rayCamPosY >= 60) && (rayCamPosY <= 72)){
            currentArea = 2;
        }
    }
    
   
    if (currentArea != previousArea){

        engInitObjects();

        if (currentArea == 0){

            LoadFileAt(LOADER_TEXTURES_1, 0xc800);
            outdoorMode();

            // engAddObject(OBJ_DOOR, 0, 21, innerDoorData);
            engObjType = OBJ_DOOR;
            engObjX     = 0;
            engObjY     = 21;
            engObjData  = innerDoorData;
            engAddObjectASM();
            objTexture[0] = 0;

            // engAddObject(OBJ_DOOR, 0, 57, outterDoorData);
            engObjType = OBJ_DOOR;
            engObjX     = 0;
            engObjY     = 57;
            engObjData  = outterDoorData;
            engAddObjectASM();
            objTexture[1] = 0;

            // engAddObject(OBJ_LETTER, 45, 0, 0);
            engObjType = OBJ_LETTER;
            engObjX     = 57;
            engObjY     = 0;
            engObjData  = 0;
            engAddObjectASM();
            objTexture[2] = texture_02;

            // engAddObject(OBJ_KEY, 57, 0, 0);
            engObjType = OBJ_KEY;
            engObjX     = 45;
            engObjY     = 0;
            engObjData  = 0;
            engAddObjectASM();
            objTexture[3] = texture_01;

            // engAddObject(OBJ_TREE, 0, 0, 0);
            engObjType = OBJ_TREE;
            engObjX     = 0;
            engObjY     = 0;
            engAddObjectASM();
            objTexture[4] = texture_03;

            // engAddObject(OBJ_TREE, 12, 12, 0);
            engObjX     = 12;
            engObjY     = 12;
            engAddObjectASM();
            objTexture[5] = texture_03;

            // engAddObject(OBJ_TREE, 8, 10, 0);
            engObjX     = 8;
            engObjY     = 10;
            engAddObjectASM();
            objTexture[6] = texture_04;

            // engAddObject(OBJ_TREE, -8, 8, 0);
            engObjX     = -8;
            engObjY     = 8;
            engAddObjectASM();
            objTexture[7] = texture_05;

            // engAddObject(OBJ_TREE, 8, -8, 0);
            engObjX     = 8;
            engObjY     = -8;
            engAddObjectASM();
            objTexture[8] = texture_06;

            // engAddObject(OBJ_TREE, -8, -8, 0);
            engObjX     = -8;
            engObjY     = -8;
            engAddObjectASM();
            objTexture[9] = texture_07;

        } else if (currentArea == 1) {

            LoadFileAt(LOADER_TEXTURES_2, 0xc800);
            indoorMode();


            // engAddObject(OBJ_DOOR, 0, 21, innerDoorData);
            engObjType = OBJ_DOOR;
            engObjX     = 0;
            engObjY     = 21;
            engObjData  = innerDoorData;
            engAddObjectASM();
            objTexture[0] = 0;

            // engAddObject(OBJ_DOOR, 0, 57, outterDoorData);
            engObjType = OBJ_DOOR;
            engObjX     = 0;
            engObjY     = 57;
            engObjData  = outterDoorData;
            engAddObjectASM();
            objTexture[1] = 0;

            // engAddObject(OBJ_CREATURE, 0, 54, 0);
            engObjType = OBJ_CREATURE;
            engObjX     = 0;
            engObjY     = 54;
            engAddObjectASM();
            objTexture[2] = texture_01;

            // engAddObject(OBJ_ARBALETE, 0, 36, 0);
            engObjType = OBJ_ARBALETE;
            engObjX     = 0;
            engObjY     = 36;
            engAddObjectASM();
            objTexture[3] = texture_02;

            // engAddObject(OBJ_CREATURE, -15, 48, 0);
            engObjType = OBJ_CREATURE;
            engObjX     = -15;
            engObjY     = 48;
            engAddObjectASM();
            objTexture[4] = texture_03;

            // engAddObject(OBJ_CREATURE, 15, 48, 0);
            engObjX     = 15;
            engObjY     = 48;
            engAddObjectASM();
            objTexture[5] = texture_04;

            // engAddObject(OBJ_CREATURE, 15, 42, 0);
            engObjX     = 15;
            engObjY     = 42;
            engAddObjectASM();
            objTexture[6] = texture_05;

            // engAddObject(OBJ_CREATURE, -15, 42, 0);
            engObjX     = -15;
            engObjY     = 42;
            engAddObjectASM();
            objTexture[7] = texture_06;

        } else if (currentArea == 2) {

            LoadFileAt(LOADER_TEXTURES_3, 0xc800);
            indoorMode();

            // engAddObject(OBJ_DOOR, 0, 21, innerDoorData);
            engObjType = OBJ_DOOR;
            engObjX     = 0;
            engObjY     = 21;
            engObjData  = innerDoorData;
            engAddObjectASM();
            objTexture[0] = 0;

            // engAddObject(OBJ_DOOR, 0, 57, outterDoorData);
            engObjType = OBJ_DOOR;
            engObjX     = 0;
            engObjY     = 57;
            engObjData  = outterDoorData;
            engAddObjectASM();
            objTexture[1] = 0;

            // engAddObject(OBJ_SOLDIER, 6, 66, soldierData);
            engObjType = OBJ_SOLDIER;
            engObjX     = 8;
            engObjY     = 66;
            engObjData  = soldierData;
            engAddObjectASM();
            objTexture[2] = texture_01;

            // engAddObject(OBJ_FIRE, 12 , 70, 0);
            engObjType = OBJ_FIRE;
            engObjX     = 12;
            engObjY     = 70;
            engObjData  = 0;
            engAddObjectASM();
            objTexture[3] = texture_08;

            // engAddObject(OBJ_FIRE, 12 , 62, 0);
            engObjX     = 12;
            engObjY     = 62;
            engAddObjectASM();
            objTexture[4] = texture_06;

            // engAddObject(OBJ_FIRE, -12 , 70, 0);
            engObjX     = -12;
            engObjY     = 70;
            engAddObjectASM();
            objTexture[5] = texture_07;

            // engAddObject(OBJ_FIRE, -12 , 62, 0);
            engObjX     = -12;
            engObjY     = 62;
            engAddObjectASM();
            objTexture[6] = texture_05;
        }
    }
}

void gameInit(void){

    // LoadFileAt(LOADER_RAYTABLES, 0xED95);
	// LoadFileAt(BASIC_SCREEN, 0xBB80);
    // LoadFileAt(LOADER_CHARSET_STANDARD, 0xb400);
    LoadFileAt(LOADER_RAYTABLES, 0xEC00);
    // LoadFileAt(LOADER_TEXTURES_3, 0xc000);
    LoadFileAt(LOADER_TEXTURES, 0xc000);
    // LoadFileAt(LOADER_ANIM, 0xd800);
    LoadFileAt(LOADER_HRSCREEN, 0xA000);
    GenerateTables();
    // prepare_screen_layout();
    initCamera();


    // [ref scene_load]
    initScene (scene_00, texture_00);

    updateCurrentArea ();

    hasKey        = 0;
    hasArbalete   = 0;
    health        = 100;
    gameOngoing   = 1;
    isEquiped     = 0;


    missileInit();
    PRINTS(3,17,"Sante: ");
}

void gamePulse(void){


    // init Shapes
    glNbVertices     = 0;
    glNbSegments   = 0;
    glNbFaces      = 0;
    glNbParticles = 0;

    // glLoadShape(
    //     -3, 6, -3, 
    //     1, 
    //     geomPlan
    // );
    // glLoadShape(
    //     3, 6, -3, 
    //     1, 
    //     geomPlan
    // );

    updateCurrentArea ();
    missilePulse();

    dichoInit();
    engPulse();

    rayInitCasting();

    rayProcessPoints();
    rayProcessWalls();
    drawWalls();
#define V_TARGET_SIZE 30
#define H_TARGET_SIZE 30
#define V_CENTER 66
#define H_CENTER 106

    if (isEquiped != 0){
        // Left UP
        LargeX0 = 8;
        LargeY0 = 1;
        LargeX1 = H_CENTER-H_TARGET_SIZE;
        LargeY1 = V_CENTER-H_TARGET_SIZE;
        DrawClippedLine();
        
        // Right UP
        LargeX0 = 219;
        LargeY0 = 1;
        LargeX1 = H_CENTER+H_TARGET_SIZE;
        LargeY1 = V_CENTER-H_TARGET_SIZE;
        DrawClippedLine();
        
        // Left BOTTOM
        LargeX0 = 8;
        LargeY0 = 131;
        LargeX1 = H_CENTER-H_TARGET_SIZE;
        LargeY1 = V_CENTER+H_TARGET_SIZE;
        DrawClippedLine();
        
        // Right BOTTOM
        LargeX0 = 219;
        LargeY0 = 131;
        LargeX1 = H_CENTER+H_TARGET_SIZE;
        LargeY1 = V_CENTER+H_TARGET_SIZE;
        DrawClippedLine();
        
    }

    glProjectArrays();
    drawSegments();
    
    if (health <= 0) {
       health = 0;
       gameOngoing = 0;
       PRINTS(11,26,"Perdu :-(");
    }
    if (gameOngoing != 0) PRINTN(10,17,health);
}
