from PIL import Image, ImageEnhance
import os
import codegen
def process_animation(image_path, output_folder):
    # Vérifie si le dossier de sortie existe, sinon crée-le
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)
    
    # Récupère le nom de base du fichier d'entrée sans l'extension
    base_name = os.path.splitext(os.path.basename(image_path))[0]
    
    # Charge le fichier (GIF ou WebP) et initialise le compteur de trame
    with Image.open(image_path) as img:
        frame_number = 0

        try:
            while True:
                # Charge la trame actuelle
                frame = img.convert("RGBA")  # Convertir en RGBA pour gérer la transparence

                # Appliquer l'ajustement de contraste
                enhancer = ImageEnhance.Contrast(frame)
                frame = enhancer.enhance(1.5)  # Augmente le contraste par 1.5x (ajustable)

                # Remplace la transparence par du rose (RGB = 152, 0, 136)
                new_frame = Image.new("RGBA", frame.size, (152, 0, 136, 255))
                new_frame.paste(frame, (0, 0), mask=frame)

                # Redimensionne en 32x32 pixels
                new_frame = new_frame.resize((32, 32), Image.ANTIALIAS)

                # Convertir en mode RGB avant de sauvegarder en BMP
                bmp_frame = new_frame.convert("RGB")

                # Sauvegarde la trame au format BMP en utilisant le nom de fichier d'entrée
                output_path = os.path.join(output_folder, f"{base_name}_frame_{frame_number}.bmp")
                bmp_frame.save(output_path)

                # Passer à la trame suivante
                frame_number += 1
                img.seek(img.tell() + 1)

        except EOFError:
            # Fin de l'animation atteinte
            print(f"{frame_number} frames processed and saved in {output_folder}")
from PIL import Image, ImageEnhance
import os

def image2TextureBuf(filepathname):
    base = os.path.basename(filepathname)
    namerad = os.path.splitext(base)[0]
    im = Image.open(filepathname)
    rgb_im = im.convert('RGB')
    imw, imh = im.width, im.height
    bufimgtranslat = []

    for ii in range(imw):
        for jj in range(imh):    
            # Convertit chaque pixel en valeur texel
            r, g, b = rgb_im.getpixel((ii, jj))
            texel_value = (r // 64) * 16 + (g // 64) * 4 + (b // 64)
            bufimgtranslat.append(texel_value)

    # Génère le code C
    cCode = codegen.buffer2cCode("texture_" + namerad, "unsigned char", bufimgtranslat)
    return cCode

def process_animation(image_path, output_folder):
    # Vérifie si le dossier de sortie existe, sinon crée-le
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)
    
    # Récupère le nom de base du fichier d'entrée sans l'extension
    base_name = os.path.splitext(os.path.basename(image_path))[0]
    
    # Charge le fichier (GIF ou WebP) et initialise le compteur de trame
    with Image.open(image_path) as img:
        frame_number = 0

        try:
            while True:
                # Charge la trame actuelle
                frame = img.convert("RGBA")  # Convertir en RGBA pour gérer la transparence

                # Appliquer l'ajustement de contraste
                enhancer = ImageEnhance.Contrast(frame)
                frame = enhancer.enhance(1.5)  # Augmente le contraste par 1.5x (ajustable)

                # Remplace la transparence par du rose (RGB = 152, 0, 136)
                new_frame = Image.new("RGBA", frame.size, (152, 0, 136, 255))
                new_frame.paste(frame, (0, 0), mask=frame)

                # Redimensionne en 32x32 pixels
                new_frame = new_frame.resize((32, 32), Image.ANTIALIAS)

                # Convertir en mode RGB avant de sauvegarder en BMP
                bmp_frame = new_frame.convert("RGB")

                # Sauvegarde la trame au format BMP en utilisant le nom de fichier d'entrée
                output_path = os.path.join(output_folder, f"{base_name}_frame_{frame_number}.bmp")
                bmp_frame.save(output_path)

                # Traitement de l'image avec image2TextureBuf pour générer le code C
                c_code = image2TextureBuf(output_path)
                
                # # Affiche ou sauvegarde le code C si nécessaire
                # print(f"Code C pour la trame {frame_number}:\n", c_code)

                # Passer à la trame suivante
                frame_number += 1
                img.seek(img.tell() + 1)

        except EOFError:
            # Fin de l'animation atteinte
            print(f"{frame_number} frames processed and saved in {output_folder}")

# Utilisation du script
image_path = "assets/fire.gif"  # Chemin du fichier GIF ou WebP d'entrée
output_folder = "data"  # Dossier de sortie
process_animation(image_path, output_folder)

image_path = "assets/dog.webp"  # Chemin du fichier GIF ou WebP d'entrée
output_folder = "data"  # Dossier de sortie
process_animation(image_path, output_folder)
